var request = require('request');
var http = require('http');
var url = require('url');
var fs = require('fs');
var ini = require('ini');
var strftime = require('strftime');
var xmpp = require('simple-xmpp');
var winston = require('winston');
require('winston-redis').Redis;

var warming_up = true;

var config = ini.parse(fs.readFileSync('./config.ini', 'utf-8'));

var logger = new (winston.Logger)({
  transports: [
    new (winston.transports.Console)({
      timestamp: function() {
        return strftime('%F %T', new Date());
      }
    }),
    new (winston.transports.DailyRotateFile)({
      filename: 'log',
      timestamp: function() {
        return strftime('%F %T', new Date());
      }
    })
  ]
});

if (config.redis && config.redis.host) {
    var winstonRedisOptions = {
        host: config.redis.host,
        port: config.redis.port || 6379,
        container: 'LOG_CENTER_' + config.globals.smscid + '_' + config.globals.jid,
        channel: 'LOG_CENTER_' + config.globals.smscid + '_' + config.globals.jid
    }
    logger.add(winston.transports.Redis, winstonRedisOptions);
    logger.info("Add redis log channel", {options: winstonRedisOptions});
}
else {
    logger.info("Skip create redis log channel because of undefined config.redis.host");
}


function createHttpListener() {

    var httpServer = http.createServer(function(request,response){

    var qs = url.parse(request.url, true).query;
        logger.info('Incoming message from SMSIN server', {qs: qs});
        response.end('OK');

        sendMessage(qs.PhoneNumber, qs.text);
    });

    httpServer.listen(config.globals.listen_port, function(){
        logger.info("HTTP server listening on " + config.globals.listen_port);
    })
}

function isInIgnoreList(sender) {
    if (isMe(sender)) {
        return true;
    }

    if (!config || !config.globals || (sender == config.globals.keepalive_id)) {
        return true;
    }

    if (!config || !config.globals || !config.globals.ignore_list) {
        return false;
    }

    var ignoreList = config.globals.ignore_list.trim().toLowerCase().split(/\s*\W*\s*/);
    return (ignoreList.indexOf(sender.toLowerCase()) >= 0);
}

function sendKeepAliveMessage() {
    if (warming_up) {
        return;
    }

    if (!config || !config.globals || !config.globals.keepalive_id || !config.globals.keepalive_message) {
        return;
    }

    sendMessage(config.globals.keepalive_id, config.globals.keepalive_message);
}

function sendPingToMe() {
    if (warming_up) {
        return;

    }
    sendMessage(config.globals.jid.replace(/\/.*/, ''), 'PING ' + strftime('%F %T', new Date()));
}

function isMe(id) {
    return id.replace(/\/.*/, '').toLowerCase() == config.globals.jid.replace(/\/.*/, '').toLowerCase();
}

function sendMessage(destination, message) {
    if (!isMe(destination)) {
        logger.info('Sending message via XMPP', {destination: destination, msg: message});
    }

    xmpp.send(destination, message);
}

function reportToSMSIN(from, message) {
    var opts = {
        url: config.globals.smsin,
        localAddress: config.globals.request_local_address,
        qs: {
            PhoneNumber: from,
            Text: message,
            Res_Port: config.globals.listen_port,
            SMSCID: config.globals.smscid
        }
    }

    request(opts, function(err, response, body) {
        if (err) {
            logger.warn('Error reporting to SMSIN: ' + err);
            return;
        }
    });
}

xmpp.on('online', function(data) {
    logger.info('XMPP online', {data: data});
    logger.info('Connected with JID: ' + data.jid.user);

    setTimeout(function() {
        warming_up = false;
        logger.info('BOT selesai warming up, pesan-pesan selanjutnya akan diproses');
    }, config.globals.warming_up);
});

xmpp.on('chat', function(from, message) {
    if (isInIgnoreList(from)) {
        logger.verbose('Incoming message via XMPP from someone in ignore list', {from: from, msg: message});
        return;
    }

    logger.info('Incoming message via XMPP ', {from: from, msg: message});

    if (warming_up) {
        logger.info('BOT masih dalam tahap warming up, abaikan pesan');
        var response_message = 'Pesan diabaikan. Silahkan diulang kembali: ' + message;
        sendMessage(from, response_message);
        return;
    }

    var response_message = 'Pesan anda telah diterima dan akan segera diproses: ' + message;
    sendMessage(from, response_message);

    reportToSMSIN(from, message);
});

xmpp.on('error', function(err) {
    logger.warn('XMPP error, terminating in 3 secs', {err: err});
    setTimeout(process.exit, 3000, 1);
});

xmpp.on('subscribe', function(from) {
    logger.warn('XMPP subscribe request from ' + from);
    xmpp.acceptSubscription(from);
});

xmpp.connect({
        jid: config.globals.jid,
        password: config.globals.password,
});

// check for incoming subscription requests
xmpp.getRoster();

createHttpListener();

if (config && config.globals && config.globals.keepalive_interval_ms) {
    logger.verbose('sendKeepAliveMessage every ' + config.globals.keepalive_interval_ms + 'ms')
    //setInterval(sendKeepAliveMessage, config.globals.keepalive_interval_ms);
    setInterval(sendPingToMe, 60 * 1000);
}
